package com.sample;

import static org.junit.Assert.assertEquals;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.core.marshalling.impl.ClassObjectMarshallingStrategyAcceptor;
import org.drools.core.marshalling.impl.SerializablePlaceholderResolverStrategy;
import org.drools.persistence.jpa.marshaller.JPAPlaceholderResolverStrategy;
import org.h2.tools.Server;
import org.jbpm.runtime.manager.impl.RuntimeEnvironmentBuilder;
import org.jbpm.services.task.identity.JBossUserGroupCallbackImpl;
import org.jbpm.services.task.utils.ContentMarshallerHelper;
import org.jbpm.test.JBPMHelper;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.io.ResourceType;
import org.kie.api.marshalling.ObjectMarshallingStrategy;
import org.kie.api.runtime.EnvironmentName;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.manager.RuntimeEnvironment;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.UserGroupCallback;
import org.kie.api.task.model.Content;
import org.kie.api.task.model.Task;
import org.kie.api.task.model.TaskSummary;
import org.kie.internal.io.ResourceFactory;
import org.kie.internal.runtime.manager.context.ProcessInstanceIdContext;

import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * This is a sample file to launch a process.
 */
public class ProcessJPATest {

	private static EntityManagerFactory emf;

	private static Server h2Server;
	private static PoolingDataSource ds;

	@Before
	public void setup() {

		// for H2 datasource
		h2Server = JBPMHelper.startH2Server();
		ds = JBPMHelper.setupDataSource();

		emf = Persistence.createEntityManagerFactory("org.jbpm.example");

	}

	@After
	public void teardown() {
		if (ds != null) {
			ds.close();
		}
		if (h2Server != null) {
			h2Server.shutdown();
		}
	}

	@Test
	public void testProcess() throws Exception {

		try {

			RuntimeManager manager = getRuntimeManager("sample.bpmn");
			RuntimeEngine runtime = manager.getRuntimeEngine(ProcessInstanceIdContext.get());
			KieSession ksession = runtime.getKieSession();

			// start a new process instance
			Map<String, Object> params = new HashMap<String, Object>();
			Application application = new Application();
			application.setType("A");
			params.put("application", application);
			ProcessInstance pi = ksession.startProcess("com.sample.bpmn.hello", params);
			System.out.println("A process instance started : pid = " + pi.getId());

			TaskService taskService = runtime.getTaskService();

			// ------------
			{
				List<TaskSummary> list = taskService.getTasksAssignedAsPotentialOwner("john", "en-UK");
				for (TaskSummary taskSummary : list) {
					System.out.println("john starts a task : taskId = " + taskSummary.getId());

					Task task = taskService.getTaskById(taskSummary.getId());
					long documentContentId = task.getTaskData().getDocumentContentId();
					Content content = taskService.getContentById(documentContentId);
					HashMap<String, Object> contents = (HashMap<String, Object>) ContentMarshallerHelper
							.unmarshall(content.getContent(), ksession.getEnvironment());

					Application outputApplication = (Application) contents.get("input1_application");
					Person person = new Person();
					person.setFullName("John Doe");
					outputApplication.setPerson(person);

					Map<String, Object> results = new HashMap<String, Object>();
					results.put("output1_application", outputApplication);

					taskService.start(taskSummary.getId(), "john");
					taskService.complete(taskSummary.getId(), "john", results);
				}
			}

			// -----------
			{
				List<TaskSummary> list = taskService.getTasksAssignedAsPotentialOwner("mary", "en-UK");
				for (TaskSummary taskSummary : list) {
					System.out.println("mary starts a task : taskId = " + taskSummary.getId());
					taskService.start(taskSummary.getId(), "mary");
					taskService.complete(taskSummary.getId(), "mary", null);
				}
			}

			// -----------
			manager.disposeRuntimeEngine(runtime);

			// Check!
			EntityManager em = emf.createEntityManager();
			int size = em.createQuery("select i from Person i").getResultList().size();
			assertEquals(1, size);

		} catch (Throwable th) {
			th.printStackTrace();
		}
	}

	private static RuntimeManager getRuntimeManager(String process) {
		Properties properties = new Properties();
		properties.setProperty("krisv", "");
		properties.setProperty("mary", "");
		properties.setProperty("john", "");
		UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);

		RuntimeEnvironment environment = RuntimeEnvironmentBuilder.getDefault().persistence(true)
				.entityManagerFactory(emf)
				.addEnvironmentEntry(EnvironmentName.OBJECT_MARSHALLING_STRATEGIES,
						new ObjectMarshallingStrategy[] { new JPAPlaceholderResolverStrategy(emf),
								new SerializablePlaceholderResolverStrategy(
										ClassObjectMarshallingStrategyAcceptor.DEFAULT) })
				.userGroupCallback(userGroupCallback)
				.addAsset(ResourceFactory.newClassPathResource(process), ResourceType.BPMN2).get();
		return RuntimeManagerFactory.Factory.get().newPerProcessInstanceRuntimeManager(environment);

	}

}